JSB.newAddon = function(mainPath) {
    // baseUrl can be modified as needed.
    // Google: http://www.google.com/search?q={keyword}
    // Baidu: http://www.baidu.com/s?wd={keyword}
    // the baseUrl can also be URL Scheme such as eudic://dict/{keyword}
    // eudic-de://dict/{keyword}
    var baseUrl = "eudic-fr://dict/";
    //MARK: - Customized functions
    // URL generation
    function generateUrl(keyWords) {
        // return baseUrl + keyWords.replace(/ +/g, "+");
        return baseUrl + keyWords;
    }
    // Open External Browser
    function openUrlWithExternalBrowser(url) {
        // endoce URL to utf-8. NSURL cannot handle Chinese url
        let encodedUrl = encodeURI(url);
        Application.sharedInstance().openURL(NSURL.URLWithString(encodedUrl));
    }


    function filterText(text) {
        var cnPattern = /^\p{Unified_Ideograph}{0,6}$/u;
        // 更新法语匹配模式，包含所有法语特殊字符
        var frPattern = /^[A-Za-zÀ-ÿŒœÇçÆæ\s'''`'-]+$/;
        
        // 法文 最多五个单词
        if (frPattern.test(text)) {
            // 移除多余空格并计算单词数
            let words = text.trim().split(/\s+/);
            if (words.length < 6) {
                return true;
            }
        } else if (cnPattern.test(text)) {
            // 中文 最长六个字
            // 不含空格
            return true;
        }
        return false;
    }
    //MARK - Addon Class definition
    var newAddonClass= JSB.defineClass('SearchInFrhelperAddon : JSExtension', {
        //Mark: - Instance Method Definitions
        // Window initialize
        sceneWillConnect: function() {
            NSNotificationCenter.defaultCenter().addObserverSelectorName(self, 'onPopupMenuOnSelection:', 'PopupMenuOnSelection');
            NSNotificationCenter.defaultCenter().addObserverSelectorName(self, 'onPopupMenuOnNote:', 'PopupMenuOnNote');
        },
        // Window disconnect
        sceneDidDisconnect: function() {
            NSNotificationCenter.defaultCenter().removeObserverName(self,'PopupMenuOnSelection');
            NSNotificationCenter.defaultCenter().removeObserverName(self,'PopupMenuOnNote');
        },
        // Window resign active
        sceneWillResignActive: function() {
        },
        // Window become active
        sceneDidBecomeActive: function() {
        },
        //MARK: MN behaviors
        notebookWillOpen: function(notebookid) {
            self.searchInEudicPlusIsOn = NSUserDefaults.standardUserDefaults().objectForKey('marginnote_searchineudic');
        },
        notebookWillClose: function(notebookid) {
        },
        documentDidOpen: function(docmd5) {
        },
        docmentWillClose: function(docmd5) {
        },
        controllerWillLayoutSubviews: function(controller) {
        },
        queryAddonCommandStatus: function() {
            if(Application.sharedInstance().studyController(self.window).studyMode < 3) {
                return {
                    image: 'SearchInFrhelper.png',
                    object: self,
                    selector: "toggleSearchInFrhelper:",
                    checked: self.searchInEudicPlusIsOn
                };
            }
            return null;
        },

        // Select text and open the external Browser to process the selected Text
        onPopupMenuOnSelection: function(sender) {
            if(!Application.sharedInstance().checkNotifySenderInWindow(sender,self.window)) {
                return; // Don't process message from other window
            }
            if(!self.searchInEudicPlusIsOn) {
                return;
            }

            var text = sender.userInfo.documentController.selectionText;
            if(text && text.length) {
                text = text.replace(/^\s\s*/, '').replace(/\s\s*$/, ''); //去除首尾空格
			if(text.includes("；") || text.includes(";")){
				text=text.split(/\s*[;；]\s*/)[0]
			}
                if(filterText(text)) {
                    let url = generateUrl(text);
                    openUrlWithExternalBrowser(url);
                }
            }
        },
		onPopupMenuOnNote: function(sender) {
            if(!Application.sharedInstance().checkNotifySenderInWindow(sender,self.window)) {
                return; // Don't process message from other window
            }
            if(!self.searchInEudicPlusIsOn) {
                return;
            }
            var text = sender.userInfo.note.noteTitle;
			if(!text){
				text=sender.userInfo.note.excerptText;
			}
            if(text && text.length) {
                text = text.replace(/^\s\s*/, '').replace(/\s\s*$/, ''); //去除首尾空格
			if(text.includes("；") || text.includes(";")){
				text=text.split(/\s*[;；]\s*/)[0]
			}
                if(filterText(text)) {
                    let url = generateUrl(text);
                    openUrlWithExternalBrowser(url);
                }
            }
        },
        // Add-On Switch
        toggleSearchInFrhelper: function(sender) {
            var lan = NSLocale.preferredLanguages().length ? NSLocale.preferredLanguages()[0].substring(0, 2) : 'en';
            let tips = lan === 'zh' ? 'Search In Frhelper插件已关闭' : 'Search In Frhelper is off';
            if(self.searchInEudicPlusIsOn) {
                self.searchInEudicPlusIsOn = false;
            } else {
                self.searchInEudicPlusIsOn = true;
                tips = lan === 'zh' ? 'Search In Frhelper插件已开启，将自动打开法语助手并查询选中单词' : 'Search In Frhelper is on, selected words will be searched in Frhelper';
            }
            Application.sharedInstance().showHUD(tips, self.window, 2);
            NSUserDefaults.standardUserDefaults().setObjectForKey(self.searchInEudicPlusIsOn, 'marginnote_searchineudic');
            Application.sharedInstance().studyController(self.window).refreshAddonCommands();
        },
    }, {
        //MARK: - Class Method Definitions
        addonDidConnect: function() {
        },
        addonWillDisconnect: function() {
        },
        applicationWillEnterForeground: function() {
        },
        applicationDidEnterBackground: function() {
        },
        applicationDidReceiveLocalNotification: function(notify) {
        },
    });
    return newAddonClass;
};
